import sys
import typing


def actionzone(modifier: int = 0):
    ''' Handle area action zones for mouse actions/gestures

    :param modifier: Modifier, Modifier state
    :type modifier: int
    '''

    pass


def animation_cancel(restore_frame: bool = True):
    ''' Cancel animation, returning to the original frame

    :param restore_frame: Restore Frame, Restore the frame when animation was initialized
    :type restore_frame: bool
    '''

    pass


def animation_play(reverse: bool = False, sync: bool = False):
    ''' Play animation

    :param reverse: Play in Reverse, Animation is played backwards
    :type reverse: bool
    :param sync: Sync, Drop frames to maintain framerate
    :type sync: bool
    '''

    pass


def animation_step():
    ''' Step through animation by position

    '''

    pass


def area_close():
    ''' Close selected area

    '''

    pass


def area_dupli():
    ''' Duplicate selected area into new window

    '''

    pass


def area_join(cursor: typing.List[int] = (0, 0)):
    ''' Join selected areas into new window

    :param cursor: Cursor
    :type cursor: typing.List[int]
    '''

    pass


def area_move(x: int = 0, y: int = 0, delta: int = 0):
    ''' Move selected area edges

    :param x: X
    :type x: int
    :param y: Y
    :type y: int
    :param delta: Delta
    :type delta: int
    '''

    pass


def area_options():
    ''' Operations for splitting and merging

    '''

    pass


def area_split(direction: typing.Union[str, int] = 'HORIZONTAL',
               factor: float = 0.5,
               cursor: typing.List[int] = (0, 0)):
    ''' Split selected area into new windows

    :param direction: Direction
    :type direction: typing.Union[str, int]
    :param factor: Factor
    :type factor: float
    :param cursor: Cursor
    :type cursor: typing.List[int]
    '''

    pass


def area_swap(cursor: typing.List[int] = (0, 0)):
    ''' Swap selected areas screen positions

    :param cursor: Cursor
    :type cursor: typing.List[int]
    '''

    pass


def back_to_previous():
    ''' Revert back to the original screen layout, before fullscreen area overlay

    '''

    pass


def delete():
    ''' Delete active screen

    '''

    pass


def drivers_editor_show():
    ''' Show drivers editor in a separate window

    '''

    pass


def frame_jump(end: bool = False):
    ''' Jump to first/last frame in frame range

    :param end: Last Frame, Jump to the last frame of the frame range
    :type end: bool
    '''

    pass


def frame_offset(delta: int = 0):
    ''' Move current frame forward/backward by a given number

    :param delta: Delta
    :type delta: int
    '''

    pass


def header_toggle_menus():
    ''' Expand or collapse the header pulldown menus

    '''

    pass


def info_log_show():
    ''' Show info log in a separate window

    '''

    pass


def keyframe_jump(next: bool = True):
    ''' Jump to previous/next keyframe

    :param next: Next Keyframe
    :type next: bool
    '''

    pass


def marker_jump(next: bool = True):
    ''' Jump to previous/next marker

    :param next: Next Marker
    :type next: bool
    '''

    pass


def new():
    ''' Add a new screen

    '''

    pass


def redo_last():
    ''' Display parameters for last action performed

    '''

    pass


def region_blend():
    ''' Blend in and out overlapping region

    '''

    pass


def region_context_menu():
    ''' Display region context menu

    '''

    pass


def region_flip():
    ''' Toggle the region's alignment (left/right or top/bottom)

    '''

    pass


def region_quadview():
    ''' Split selected area into camera, front, right, and top views

    '''

    pass


def region_scale():
    ''' Scale selected area

    '''

    pass


def region_toggle(region_type: typing.Union[str, int] = 'WINDOW'):
    ''' Hide or unhide the region

    :param region_type: Region Type, Type of the region to toggle
    :type region_type: typing.Union[str, int]
    '''

    pass


def repeat_history(index: int = 0):
    ''' Display menu for previous actions performed

    :param index: Index
    :type index: int
    '''

    pass


def repeat_last():
    ''' Repeat last action

    '''

    pass


def screen_full_area(use_hide_panels: bool = False):
    ''' Toggle display selected area as fullscreen/maximized

    :param use_hide_panels: Hide Panels, Hide all the panels
    :type use_hide_panels: bool
    '''

    pass


def screen_set(delta: int = 1):
    ''' Cycle through available screens

    :param delta: Delta
    :type delta: int
    '''

    pass


def screenshot(filepath: str = "",
               hide_props_region: bool = True,
               check_existing: bool = True,
               filter_blender: bool = False,
               filter_backup: bool = False,
               filter_image: bool = True,
               filter_movie: bool = False,
               filter_python: bool = False,
               filter_font: bool = False,
               filter_sound: bool = False,
               filter_text: bool = False,
               filter_archive: bool = False,
               filter_btx: bool = False,
               filter_collada: bool = False,
               filter_alembic: bool = False,
               filter_usd: bool = False,
               filter_volume: bool = False,
               filter_folder: bool = True,
               filter_blenlib: bool = False,
               filemode: int = 9,
               show_multiview: bool = False,
               use_multiview: bool = False,
               display_type: typing.Union[str, int] = 'DEFAULT',
               sort_method: typing.Union[str, int] = '',
               full: bool = True):
    ''' Capture a picture of the active area or whole Blender window

    :param filepath: File Path, Path to file
    :type filepath: str
    :param hide_props_region: Hide Operator Properties, Collapse the region displaying the operator settings
    :type hide_props_region: bool
    :param check_existing: Check Existing, Check and warn on overwriting existing files
    :type check_existing: bool
    :param filter_blender: Filter .blend files
    :type filter_blender: bool
    :param filter_backup: Filter .blend files
    :type filter_backup: bool
    :param filter_image: Filter image files
    :type filter_image: bool
    :param filter_movie: Filter movie files
    :type filter_movie: bool
    :param filter_python: Filter python files
    :type filter_python: bool
    :param filter_font: Filter font files
    :type filter_font: bool
    :param filter_sound: Filter sound files
    :type filter_sound: bool
    :param filter_text: Filter text files
    :type filter_text: bool
    :param filter_archive: Filter archive files
    :type filter_archive: bool
    :param filter_btx: Filter btx files
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files
    :type filter_alembic: bool
    :param filter_usd: Filter USD files
    :type filter_usd: bool
    :param filter_volume: Filter OpenVDB volume files
    :type filter_volume: bool
    :param filter_folder: Filter folders
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file
    :type filemode: int
    :param show_multiview: Enable Multi-View
    :type show_multiview: bool
    :param use_multiview: Use Multi-View
    :type use_multiview: bool
    :param display_type: Display Type * DEFAULT Default -- Automatically determine display type for files. * LIST_VERTICAL Short List -- Display files as short list. * LIST_HORIZONTAL Long List -- Display files as a detailed list. * THUMBNAIL Thumbnails -- Display files as thumbnails.
    :type display_type: typing.Union[str, int]
    :param sort_method: File sorting mode
    :type sort_method: typing.Union[str, int]
    :param full: Full Screen, Capture the whole window (otherwise only capture the active area)
    :type full: bool
    '''

    pass


def space_context_cycle(direction: typing.Union[str, int] = 'NEXT'):
    ''' Cycle through the editor context by activating the next/previous one

    :param direction: Direction, Direction to cycle through
    :type direction: typing.Union[str, int]
    '''

    pass


def space_type_set_or_cycle(space_type: typing.Union[str, int] = 'EMPTY'):
    ''' Set the space type or cycle subtype

    :param space_type: Type * EMPTY Empty. * VIEW_3D 3D Viewport -- Manipulate objects in a 3D environment. * IMAGE_EDITOR UV/Image Editor -- View and edit images and UV Maps. * NODE_EDITOR Node Editor -- Editor for node-based shading and compositing tools. * SEQUENCE_EDITOR Video Sequencer -- Video editing tools. * CLIP_EDITOR Movie Clip Editor -- Motion tracking tools. * DOPESHEET_EDITOR Dope Sheet -- Adjust timing of keyframes. * GRAPH_EDITOR Graph Editor -- Edit drivers and keyframe interpolation. * NLA_EDITOR Nonlinear Animation -- Combine and layer Actions. * TEXT_EDITOR Text Editor -- Edit scripts and in-file documentation. * CONSOLE Python Console -- Interactive programmatic console for advanced editing and script development. * INFO Info -- Log of operations, warnings and error messages. * TOPBAR Top Bar -- Global bar at the top of the screen for global per-window settings. * STATUSBAR Status Bar -- Global bar at the bottom of the screen for general status information. * OUTLINER Outliner -- Overview of scene graph and all available data-blocks. * PROPERTIES Properties -- Edit properties of active object and related data-blocks. * FILE_BROWSER File Browser -- Browse for files and assets. * SPREADSHEET Spreadsheet -- Explore geometry data in a table. * PREFERENCES Preferences -- Edit persistent configuration settings.
    :type space_type: typing.Union[str, int]
    '''

    pass


def spacedata_cleanup():
    ''' Remove unused settings for invisible editors

    '''

    pass


def userpref_show():
    ''' Edit user preferences and system settings

    '''

    pass


def workspace_cycle(direction: typing.Union[str, int] = 'NEXT'):
    ''' Cycle through workspaces

    :param direction: Direction, Direction to cycle through
    :type direction: typing.Union[str, int]
    '''

    pass



import sys
import typing


def edge_pan(inside_padding: float = 1.0,
             outside_padding: float = 0.0,
             speed_ramp: float = 1.0,
             max_speed: float = 500.0,
             delay: float = 1.0):
    ''' Pan the view when the mouse is held at an edge

    :param inside_padding: Inside Padding, Inside distance in UI units from the edge of the region within which to start panning
    :type inside_padding: float
    :param outside_padding: Outside Padding, Outside distance in UI units from the edge of the region at which to stop panning
    :type outside_padding: float
    :param speed_ramp: Speed Ramp, Width of the zone in UI units where speed increases with distance from the edge
    :type speed_ramp: float
    :param max_speed: Max Speed, Maximum speed in UI units per second
    :type max_speed: float
    :param delay: Delay, Delay in seconds before maximum speed is reached
    :type delay: float
    '''

    pass


def ndof():
    ''' Use a 3D mouse device to pan/zoom the view

    '''

    pass


def pan(deltax: int = 0, deltay: int = 0):
    ''' Pan the view

    :param deltax: Delta X
    :type deltax: int
    :param deltay: Delta Y
    :type deltay: int
    '''

    pass


def reset():
    ''' Reset the view

    '''

    pass


def scroll_down(deltax: int = 0, deltay: int = 0, page: bool = False):
    ''' Scroll the view down

    :param deltax: Delta X
    :type deltax: int
    :param deltay: Delta Y
    :type deltay: int
    :param page: Page, Scroll down one page
    :type page: bool
    '''

    pass


def scroll_left(deltax: int = 0, deltay: int = 0):
    ''' Scroll the view left

    :param deltax: Delta X
    :type deltax: int
    :param deltay: Delta Y
    :type deltay: int
    '''

    pass


def scroll_right(deltax: int = 0, deltay: int = 0):
    ''' Scroll the view right

    :param deltax: Delta X
    :type deltax: int
    :param deltay: Delta Y
    :type deltay: int
    '''

    pass


def scroll_up(deltax: int = 0, deltay: int = 0, page: bool = False):
    ''' Scroll the view up

    :param deltax: Delta X
    :type deltax: int
    :param deltay: Delta Y
    :type deltay: int
    :param page: Page, Scroll up one page
    :type page: bool
    '''

    pass


def scroller_activate():
    ''' Scroll view by mouse click and drag

    '''

    pass


def smoothview(xmin: int = 0,
               xmax: int = 0,
               ymin: int = 0,
               ymax: int = 0,
               wait_for_input: bool = True):
    ''' Undocumented, consider contributing <https://developer.blender.org/T51061> __.

    :param xmin: X Min
    :type xmin: int
    :param xmax: X Max
    :type xmax: int
    :param ymin: Y Min
    :type ymin: int
    :param ymax: Y Max
    :type ymax: int
    :param wait_for_input: Wait for Input
    :type wait_for_input: bool
    '''

    pass


def zoom(deltax: float = 0.0,
         deltay: float = 0.0,
         use_cursor_init: bool = True):
    ''' Zoom in/out the view

    :param deltax: Delta X
    :type deltax: float
    :param deltay: Delta Y
    :type deltay: float
    :param use_cursor_init: Use Mouse Position, Allow the initial mouse position to be used
    :type use_cursor_init: bool
    '''

    pass


def zoom_border(xmin: int = 0,
                xmax: int = 0,
                ymin: int = 0,
                ymax: int = 0,
                wait_for_input: bool = True,
                zoom_out: bool = False):
    ''' Zoom in the view to the nearest item contained in the border

    :param xmin: X Min
    :type xmin: int
    :param xmax: X Max
    :type xmax: int
    :param ymin: Y Min
    :type ymin: int
    :param ymax: Y Max
    :type ymax: int
    :param wait_for_input: Wait for Input
    :type wait_for_input: bool
    :param zoom_out: Zoom Out
    :type zoom_out: bool
    '''

    pass


def zoom_in(zoomfacx: float = 0.0, zoomfacy: float = 0.0):
    ''' Zoom in the view

    :param zoomfacx: Zoom Factor X
    :type zoomfacx: float
    :param zoomfacy: Zoom Factor Y
    :type zoomfacy: float
    '''

    pass


def zoom_out(zoomfacx: float = 0.0, zoomfacy: float = 0.0):
    ''' Zoom out the view

    :param zoomfacx: Zoom Factor X
    :type zoomfacx: float
    :param zoomfacy: Zoom Factor Y
    :type zoomfacy: float
    '''

    pass



import sys
import typing


def color_add():
    ''' Add new color to active palette

    '''

    pass


def color_delete():
    ''' Remove active color from palette

    '''

    pass


def color_move(type: typing.Union[str, int] = 'UP'):
    ''' Move the active Color up/down in the list

    :param type: Type
    :type type: typing.Union[str, int]
    '''

    pass


def extract_from_image(threshold: int = 1):
    ''' Extract all colors used in Image and create a Palette

    :param threshold: Threshold
    :type threshold: int
    '''

    pass


def join(palette: str = ""):
    ''' Join Palette Swatches

    :param palette: Palette, Name of the Palette
    :type palette: str
    '''

    pass


def new():
    ''' Add new palette

    '''

    pass


def sort(type: typing.Union[str, int] = 'HSV'):
    ''' Sort Palette Colors

    :param type: Type
    :type type: typing.Union[str, int]
    '''

    pass



import sys
import typing


def armature_apply(selected: bool = False):
    ''' Apply the current pose as the new rest pose

    :param selected: Selected Only, Only apply the selected bones (with propagation to children)
    :type selected: bool
    '''

    pass


def autoside_names(axis: typing.Union[str, int] = 'XAXIS'):
    ''' Automatically renames the selected bones according to which side of the target axis they fall on

    :param axis: Axis, Axis tag names with * XAXIS X-Axis -- Left/Right. * YAXIS Y-Axis -- Front/Back. * ZAXIS Z-Axis -- Top/Bottom.
    :type axis: typing.Union[str, int]
    '''

    pass


def bone_layers(
        layers: typing.List[bool] = (False, False, False, False, False, False,
                                     False, False, False, False, False, False,
                                     False, False, False, False, False, False,
                                     False, False, False, False, False, False,
                                     False, False, False, False, False, False,
                                     False, False)):
    ''' Change the layers that the selected bones belong to

    :param layers: Layer, Armature layers that bone belongs to
    :type layers: typing.List[bool]
    '''

    pass


def breakdown(factor: float = 0.5,
              prev_frame: int = 0,
              next_frame: int = 0,
              channels: typing.Union[str, int] = 'ALL',
              axis_lock: typing.Union[str, int] = 'FREE'):
    ''' Create a suitable breakdown pose on the current frame

    :param factor: Factor, Weighting factor for which keyframe is favored more
    :type factor: float
    :param prev_frame: Previous Keyframe, Frame number of keyframe immediately before the current frame
    :type prev_frame: int
    :param next_frame: Next Keyframe, Frame number of keyframe immediately after the current frame
    :type next_frame: int
    :param channels: Channels, Set of properties that are affected * ALL All Properties -- All properties, including transforms, bendy bone shape, and custom properties. * LOC Location -- Location only. * ROT Rotation -- Rotation only. * SIZE Scale -- Scale only. * BBONE Bendy Bone -- Bendy Bone shape properties. * CUSTOM Custom Properties -- Custom properties.
    :type channels: typing.Union[str, int]
    :param axis_lock: Axis Lock, Transform axis to restrict effects to * FREE Free -- All axes are affected. * X X -- Only X-axis transforms are affected. * Y Y -- Only Y-axis transforms are affected. * Z Z -- Only Z-axis transforms are affected.
    :type axis_lock: typing.Union[str, int]
    '''

    pass


def constraint_add(type: typing.Union[str, int] = ''):
    ''' Add a constraint to the active bone

    :param type: Type * CAMERA_SOLVER Camera Solver. * FOLLOW_TRACK Follow Track. * OBJECT_SOLVER Object Solver. * COPY_LOCATION Copy Location -- Copy the location of a target (with an optional offset), so that they move together. * COPY_ROTATION Copy Rotation -- Copy the rotation of a target (with an optional offset), so that they rotate together. * COPY_SCALE Copy Scale -- Copy the scale factors of a target (with an optional offset), so that they are scaled by the same amount. * COPY_TRANSFORMS Copy Transforms -- Copy all the transformations of a target, so that they move together. * LIMIT_DISTANCE Limit Distance -- Restrict movements to within a certain distance of a target (at the time of constraint evaluation only). * LIMIT_LOCATION Limit Location -- Restrict movement along each axis within given ranges. * LIMIT_ROTATION Limit Rotation -- Restrict rotation along each axis within given ranges. * LIMIT_SCALE Limit Scale -- Restrict scaling along each axis with given ranges. * MAINTAIN_VOLUME Maintain Volume -- Compensate for scaling one axis by applying suitable scaling to the other two axes. * TRANSFORM Transformation -- Use one transform property from target to control another (or same) property on owner. * TRANSFORM_CACHE Transform Cache -- Look up the transformation matrix from an external file. * CLAMP_TO Clamp To -- Restrict movements to lie along a curve by remapping location along curve's longest axis. * DAMPED_TRACK Damped Track -- Point towards a target by performing the smallest rotation necessary. * IK Inverse Kinematics -- Control a chain of bones by specifying the endpoint target (Bones only). * LOCKED_TRACK Locked Track -- Rotate around the specified ('locked') axis to point towards a target. * SPLINE_IK Spline IK -- Align chain of bones along a curve (Bones only). * STRETCH_TO Stretch To -- Stretch along Y-Axis to point towards a target. * TRACK_TO Track To -- Legacy tracking constraint prone to twisting artifacts. * ACTION Action -- Use transform property of target to look up pose for owner from an Action. * ARMATURE Armature -- Apply weight-blended transformation from multiple bones like the Armature modifier. * CHILD_OF Child Of -- Make target the 'detachable' parent of owner. * FLOOR Floor -- Use position (and optionally rotation) of target to define a 'wall' or 'floor' that the owner can not cross. * FOLLOW_PATH Follow Path -- Use to animate an object/bone following a path. * PIVOT Pivot -- Change pivot point for transforms (buggy). * SHRINKWRAP Shrinkwrap -- Restrict movements to surface of target mesh.
    :type type: typing.Union[str, int]
    '''

    pass


def constraint_add_with_targets(type: typing.Union[str, int] = ''):
    ''' Add a constraint to the active bone, with target (where applicable) set to the selected Objects/Bones

    :param type: Type * CAMERA_SOLVER Camera Solver. * FOLLOW_TRACK Follow Track. * OBJECT_SOLVER Object Solver. * COPY_LOCATION Copy Location -- Copy the location of a target (with an optional offset), so that they move together. * COPY_ROTATION Copy Rotation -- Copy the rotation of a target (with an optional offset), so that they rotate together. * COPY_SCALE Copy Scale -- Copy the scale factors of a target (with an optional offset), so that they are scaled by the same amount. * COPY_TRANSFORMS Copy Transforms -- Copy all the transformations of a target, so that they move together. * LIMIT_DISTANCE Limit Distance -- Restrict movements to within a certain distance of a target (at the time of constraint evaluation only). * LIMIT_LOCATION Limit Location -- Restrict movement along each axis within given ranges. * LIMIT_ROTATION Limit Rotation -- Restrict rotation along each axis within given ranges. * LIMIT_SCALE Limit Scale -- Restrict scaling along each axis with given ranges. * MAINTAIN_VOLUME Maintain Volume -- Compensate for scaling one axis by applying suitable scaling to the other two axes. * TRANSFORM Transformation -- Use one transform property from target to control another (or same) property on owner. * TRANSFORM_CACHE Transform Cache -- Look up the transformation matrix from an external file. * CLAMP_TO Clamp To -- Restrict movements to lie along a curve by remapping location along curve's longest axis. * DAMPED_TRACK Damped Track -- Point towards a target by performing the smallest rotation necessary. * IK Inverse Kinematics -- Control a chain of bones by specifying the endpoint target (Bones only). * LOCKED_TRACK Locked Track -- Rotate around the specified ('locked') axis to point towards a target. * SPLINE_IK Spline IK -- Align chain of bones along a curve (Bones only). * STRETCH_TO Stretch To -- Stretch along Y-Axis to point towards a target. * TRACK_TO Track To -- Legacy tracking constraint prone to twisting artifacts. * ACTION Action -- Use transform property of target to look up pose for owner from an Action. * ARMATURE Armature -- Apply weight-blended transformation from multiple bones like the Armature modifier. * CHILD_OF Child Of -- Make target the 'detachable' parent of owner. * FLOOR Floor -- Use position (and optionally rotation) of target to define a 'wall' or 'floor' that the owner can not cross. * FOLLOW_PATH Follow Path -- Use to animate an object/bone following a path. * PIVOT Pivot -- Change pivot point for transforms (buggy). * SHRINKWRAP Shrinkwrap -- Restrict movements to surface of target mesh.
    :type type: typing.Union[str, int]
    '''

    pass


def constraints_clear():
    ''' Clear all the constraints for the selected bones

    '''

    pass


def constraints_copy():
    ''' Copy constraints to other selected bones

    '''

    pass


def copy():
    ''' Copies the current pose of the selected bones to copy/paste buffer

    '''

    pass


def flip_names(do_strip_numbers: bool = False):
    ''' Flips (and corrects) the axis suffixes of the names of selected bones

    :param do_strip_numbers: Strip Numbers, Try to remove right-most dot-number from flipped names. Warning: May result in incoherent naming in some cases
    :type do_strip_numbers: bool
    '''

    pass


def group_add():
    ''' Add a new bone group

    '''

    pass


def group_assign(type: int = 0):
    ''' Add selected bones to the chosen bone group

    :param type: Bone Group Index
    :type type: int
    '''

    pass


def group_deselect():
    ''' Deselect bones of active Bone Group

    '''

    pass


def group_move(direction: typing.Union[str, int] = 'UP'):
    ''' Change position of active Bone Group in list of Bone Groups

    :param direction: Direction, Direction to move the active Bone Group towards
    :type direction: typing.Union[str, int]
    '''

    pass


def group_remove():
    ''' Remove the active bone group

    '''

    pass


def group_select():
    ''' Select bones in active Bone Group

    '''

    pass


def group_sort():
    ''' Sort Bone Groups by their names in ascending order

    '''

    pass


def group_unassign():
    ''' Remove selected bones from all bone groups

    '''

    pass


def hide(unselected: bool = False):
    ''' Tag selected bones to not be visible in Pose Mode

    :param unselected: Unselected
    :type unselected: bool
    '''

    pass


def ik_add(with_targets: bool = True):
    ''' Add IK Constraint to the active Bone

    :param with_targets: With Targets, Assign IK Constraint with targets derived from the select bones/objects
    :type with_targets: bool
    '''

    pass


def ik_clear():
    ''' Remove all IK Constraints from selected bones

    '''

    pass


def loc_clear():
    ''' Reset locations of selected bones to their default values

    '''

    pass


def paste(flipped: bool = False, selected_mask: bool = False):
    ''' Paste the stored pose on to the current pose

    :param flipped: Flipped on X-Axis, Paste the stored pose flipped on to current pose
    :type flipped: bool
    :param selected_mask: On Selected Only, Only paste the stored pose on to selected bones in the current pose
    :type selected_mask: bool
    '''

    pass


def paths_calculate(start_frame: int = 1,
                    end_frame: int = 250,
                    bake_location: typing.Union[str, int] = 'HEADS'):
    ''' Calculate paths for the selected bones

    :param start_frame: Start, First frame to calculate bone paths on
    :type start_frame: int
    :param end_frame: End, Last frame to calculate bone paths on
    :type end_frame: int
    :param bake_location: Bake Location, Which point on the bones is used when calculating paths * HEADS Heads -- Calculate bone paths from heads. * TAILS Tails -- Calculate bone paths from tails.
    :type bake_location: typing.Union[str, int]
    '''

    pass


def paths_clear(only_selected: bool = False):
    ''' Clear path caches for all bones, hold Shift key for selected bones only

    :param only_selected: Only Selected, Only clear paths from selected bones
    :type only_selected: bool
    '''

    pass


def paths_range_update():
    ''' Update frame range for motion paths from the Scene's current frame range

    '''

    pass


def paths_update():
    ''' Recalculate paths for bones that already have them

    '''

    pass


def propagate(mode: typing.Union[str, int] = 'WHILE_HELD',
              end_frame: float = 250.0):
    ''' Copy selected aspects of the current pose to subsequent poses already keyframed

    :param mode: Terminate Mode, Method used to determine when to stop propagating pose to keyframes * WHILE_HELD While Held -- Propagate pose to all keyframes after current frame that don't change (Default behavior). * NEXT_KEY To Next Keyframe -- Propagate pose to first keyframe following the current frame only. * LAST_KEY To Last Keyframe -- Propagate pose to the last keyframe only (i.e. making action cyclic). * BEFORE_FRAME Before Frame -- Propagate pose to all keyframes between current frame and 'Frame' property. * BEFORE_END Before Last Keyframe -- Propagate pose to all keyframes from current frame until no more are found. * SELECTED_KEYS On Selected Keyframes -- Propagate pose to all selected keyframes. * SELECTED_MARKERS On Selected Markers -- Propagate pose to all keyframes occurring on frames with Scene Markers after the current frame.
    :type mode: typing.Union[str, int]
    :param end_frame: End Frame, Frame to stop propagating frames to (for 'Before Frame' mode)
    :type end_frame: float
    '''

    pass


def push(factor: float = 0.5,
         prev_frame: int = 0,
         next_frame: int = 0,
         channels: typing.Union[str, int] = 'ALL',
         axis_lock: typing.Union[str, int] = 'FREE'):
    ''' Exaggerate the current pose in regards to the breakdown pose

    :param factor: Factor, Weighting factor for which keyframe is favored more
    :type factor: float
    :param prev_frame: Previous Keyframe, Frame number of keyframe immediately before the current frame
    :type prev_frame: int
    :param next_frame: Next Keyframe, Frame number of keyframe immediately after the current frame
    :type next_frame: int
    :param channels: Channels, Set of properties that are affected * ALL All Properties -- All properties, including transforms, bendy bone shape, and custom properties. * LOC Location -- Location only. * ROT Rotation -- Rotation only. * SIZE Scale -- Scale only. * BBONE Bendy Bone -- Bendy Bone shape properties. * CUSTOM Custom Properties -- Custom properties.
    :type channels: typing.Union[str, int]
    :param axis_lock: Axis Lock, Transform axis to restrict effects to * FREE Free -- All axes are affected. * X X -- Only X-axis transforms are affected. * Y Y -- Only Y-axis transforms are affected. * Z Z -- Only Z-axis transforms are affected.
    :type axis_lock: typing.Union[str, int]
    '''

    pass


def push_rest(factor: float = 0.5,
              prev_frame: int = 0,
              next_frame: int = 0,
              channels: typing.Union[str, int] = 'ALL',
              axis_lock: typing.Union[str, int] = 'FREE'):
    ''' Push the current pose further away from the rest pose

    :param factor: Factor, Weighting factor for which keyframe is favored more
    :type factor: float
    :param prev_frame: Previous Keyframe, Frame number of keyframe immediately before the current frame
    :type prev_frame: int
    :param next_frame: Next Keyframe, Frame number of keyframe immediately after the current frame
    :type next_frame: int
    :param channels: Channels, Set of properties that are affected * ALL All Properties -- All properties, including transforms, bendy bone shape, and custom properties. * LOC Location -- Location only. * ROT Rotation -- Rotation only. * SIZE Scale -- Scale only. * BBONE Bendy Bone -- Bendy Bone shape properties. * CUSTOM Custom Properties -- Custom properties.
    :type channels: typing.Union[str, int]
    :param axis_lock: Axis Lock, Transform axis to restrict effects to * FREE Free -- All axes are affected. * X X -- Only X-axis transforms are affected. * Y Y -- Only Y-axis transforms are affected. * Z Z -- Only Z-axis transforms are affected.
    :type axis_lock: typing.Union[str, int]
    '''

    pass


def quaternions_flip():
    ''' Flip quaternion values to achieve desired rotations, while maintaining the same orientations

    '''

    pass


def relax(factor: float = 0.5,
          prev_frame: int = 0,
          next_frame: int = 0,
          channels: typing.Union[str, int] = 'ALL',
          axis_lock: typing.Union[str, int] = 'FREE'):
    ''' Make the current pose more similar to its breakdown pose

    :param factor: Factor, Weighting factor for which keyframe is favored more
    :type factor: float
    :param prev_frame: Previous Keyframe, Frame number of keyframe immediately before the current frame
    :type prev_frame: int
    :param next_frame: Next Keyframe, Frame number of keyframe immediately after the current frame
    :type next_frame: int
    :param channels: Channels, Set of properties that are affected * ALL All Properties -- All properties, including transforms, bendy bone shape, and custom properties. * LOC Location -- Location only. * ROT Rotation -- Rotation only. * SIZE Scale -- Scale only. * BBONE Bendy Bone -- Bendy Bone shape properties. * CUSTOM Custom Properties -- Custom properties.
    :type channels: typing.Union[str, int]
    :param axis_lock: Axis Lock, Transform axis to restrict effects to * FREE Free -- All axes are affected. * X X -- Only X-axis transforms are affected. * Y Y -- Only Y-axis transforms are affected. * Z Z -- Only Z-axis transforms are affected.
    :type axis_lock: typing.Union[str, int]
    '''

    pass


def relax_rest(factor: float = 0.5,
               prev_frame: int = 0,
               next_frame: int = 0,
               channels: typing.Union[str, int] = 'ALL',
               axis_lock: typing.Union[str, int] = 'FREE'):
    ''' Make the current pose more similar to the rest pose

    :param factor: Factor, Weighting factor for which keyframe is favored more
    :type factor: float
    :param prev_frame: Previous Keyframe, Frame number of keyframe immediately before the current frame
    :type prev_frame: int
    :param next_frame: Next Keyframe, Frame number of keyframe immediately after the current frame
    :type next_frame: int
    :param channels: Channels, Set of properties that are affected * ALL All Properties -- All properties, including transforms, bendy bone shape, and custom properties. * LOC Location -- Location only. * ROT Rotation -- Rotation only. * SIZE Scale -- Scale only. * BBONE Bendy Bone -- Bendy Bone shape properties. * CUSTOM Custom Properties -- Custom properties.
    :type channels: typing.Union[str, int]
    :param axis_lock: Axis Lock, Transform axis to restrict effects to * FREE Free -- All axes are affected. * X X -- Only X-axis transforms are affected. * Y Y -- Only Y-axis transforms are affected. * Z Z -- Only Z-axis transforms are affected.
    :type axis_lock: typing.Union[str, int]
    '''

    pass


def reveal(select: bool = True):
    ''' Reveal all bones hidden in Pose Mode

    :param select: Select
    :type select: bool
    '''

    pass


def rot_clear():
    ''' Reset rotations of selected bones to their default values

    '''

    pass


def rotation_mode_set(type: typing.Union[str, int] = 'QUATERNION'):
    ''' Set the rotation representation used by selected bones

    :param type: Rotation Mode * QUATERNION Quaternion (WXYZ) -- No Gimbal Lock. * XYZ XYZ Euler -- XYZ Rotation Order - prone to Gimbal Lock (default). * XZY XZY Euler -- XZY Rotation Order - prone to Gimbal Lock. * YXZ YXZ Euler -- YXZ Rotation Order - prone to Gimbal Lock. * YZX YZX Euler -- YZX Rotation Order - prone to Gimbal Lock. * ZXY ZXY Euler -- ZXY Rotation Order - prone to Gimbal Lock. * ZYX ZYX Euler -- ZYX Rotation Order - prone to Gimbal Lock. * AXIS_ANGLE Axis Angle -- Axis Angle (W+XYZ), defines a rotation around some axis defined by 3D-Vector.
    :type type: typing.Union[str, int]
    '''

    pass


def scale_clear():
    ''' Reset scaling of selected bones to their default values

    '''

    pass


def select_all(action: typing.Union[str, int] = 'TOGGLE'):
    ''' Toggle selection status of all bones

    :param action: Action, Selection action to execute * TOGGLE Toggle -- Toggle selection for all elements. * SELECT Select -- Select all elements. * DESELECT Deselect -- Deselect all elements. * INVERT Invert -- Invert selection of all elements.
    :type action: typing.Union[str, int]
    '''

    pass


def select_constraint_target():
    ''' Select bones used as targets for the currently selected bones

    '''

    pass


def select_grouped(extend: bool = False,
                   type: typing.Union[str, int] = 'LAYER'):
    ''' Select all visible bones grouped by similar properties

    :param extend: Extend, Extend selection instead of deselecting everything first
    :type extend: bool
    :param type: Type * LAYER Layer -- Shared layers. * GROUP Group -- Shared group. * KEYINGSET Keying Set -- All bones affected by active Keying Set.
    :type type: typing.Union[str, int]
    '''

    pass


def select_hierarchy(direction: typing.Union[str, int] = 'PARENT',
                     extend: bool = False):
    ''' Select immediate parent/children of selected bones

    :param direction: Direction
    :type direction: typing.Union[str, int]
    :param extend: Extend, Extend the selection
    :type extend: bool
    '''

    pass


def select_linked():
    ''' Select all bones linked by parent/child connections to the current selection

    '''

    pass


def select_linked_pick(extend: bool = False):
    ''' Select bones linked by parent/child connections under the mouse cursor

    :param extend: Extend, Extend selection instead of deselecting everything first
    :type extend: bool
    '''

    pass


def select_mirror(only_active: bool = False, extend: bool = False):
    ''' Mirror the bone selection

    :param only_active: Active Only, Only operate on the active bone
    :type only_active: bool
    :param extend: Extend, Extend the selection
    :type extend: bool
    '''

    pass


def select_parent():
    ''' Select bones that are parents of the currently selected bones

    '''

    pass


def transforms_clear():
    ''' Reset location, rotation, and scaling of selected bones to their default values

    '''

    pass


def user_transforms_clear(only_selected: bool = True):
    ''' Reset pose bone transforms to keyframed state

    :param only_selected: Only Selected, Only visible/selected bones
    :type only_selected: bool
    '''

    pass


def visual_transform_apply():
    ''' Apply final constrained position of pose bones to their transform

    '''

    pass



import sys
import typing
import bpy.types
import bpy.context

from . import units
from . import previews


def app_template_paths(*, path: str = None):
    ''' Returns valid application template paths.

    :param path: Optional subdir.
    :type path: str
    :return: app template paths.
    '''

    pass


def blend_paths(absolute: bool = False,
                packed: bool = False,
                local: bool = False) -> list:
    ''' Returns a list of paths to external files referenced by the loaded .blend file.

    :param absolute: When true the paths returned are made absolute.
    :type absolute: bool
    :param packed: When true skip file paths for packed data.
    :type packed: bool
    :param local: When true skip linked library paths.
    :type local: bool
    :rtype: list
    :return: path list.
    '''

    pass


def escape_identifier(string: str) -> str:
    ''' Simple string escaping function used for animation paths.

    :param string: text
    :type string: str
    :rtype: str
    :return: The escaped string.
    '''

    pass


def execfile(filepath: str, *, mod=None):
    ''' Execute a file path as a Python script.

    :param filepath: Path of the script to execute.
    :type filepath: str
    :param mod: Optional cached module, the result of a previous execution.
    :return: The module which can be passed back in as mod .
    '''

    pass


def is_path_builtin(path):
    ''' 

    '''

    pass


def keyconfig_init():
    ''' 

    '''

    pass


def keyconfig_set(filepath, *, report=None):
    ''' 

    '''

    pass


def load_scripts(*,
                 reload_scripts: bool = False,
                 refresh_scripts: bool = False):
    ''' Load scripts and run each modules register function.

    :param reload_scripts: Causes all scripts to have their unregister method called before loading.
    :type reload_scripts: bool
    :param refresh_scripts: only load scripts which are not already loaded as modules.
    :type refresh_scripts: bool
    '''

    pass


def make_rna_paths(struct_name: str, prop_name: str, enum_name: str) -> tuple:
    ''' Create RNA "paths" from given names.

    :param struct_name: Name of a RNA struct (like e.g. "Scene").
    :type struct_name: str
    :param prop_name: Name of a RNA struct's property.
    :type prop_name: str
    :param enum_name: Name of a RNA enum identifier.
    :type enum_name: str
    :rtype: tuple
    :return: A triple of three "RNA paths" (most_complete_path, "struct.prop", "struct.prop:'enum'"). If no enum_name is given, the third element will always be void.
    '''

    pass


def manual_map():
    ''' 

    '''

    pass


def modules_from_path(path: str, loaded_modules: set) -> list:
    ''' Load all modules in a path and return them as a list.

    :param path: this path is scanned for scripts and packages.
    :type path: str
    :param loaded_modules: already loaded module names, files matching these names will be ignored.
    :type loaded_modules: set
    :rtype: list
    :return: all loaded modules.
    '''

    pass


def preset_find(name, preset_path, *, display_name=False, ext='.py'):
    ''' 

    '''

    pass


def preset_paths(subdir: str) -> list:
    ''' Returns a list of paths for a specific preset.

    :param subdir: preset subdirectory (must not be an absolute path).
    :type subdir: str
    :rtype: list
    :return: script paths.
    '''

    pass


def refresh_script_paths():
    ''' Run this after creating new script paths to update sys.path

    '''

    pass


def register_class(cls):
    ''' Register a subclass of a Blender type class. :raises ValueError: if the class is not a subclass of a registerable blender class.

    :param cls: bpy.types.Panel , bpy.types.UIList , bpy.types.Menu , bpy.types.Header , bpy.types.Operator , bpy.types.KeyingSetInfo , bpy.types.RenderEngine
    '''

    pass


def register_classes_factory(classes):
    ''' Utility function to create register and unregister functions which simply registers and unregisters a sequence of classes.

    '''

    pass


def register_manual_map(manual_hook):
    ''' 

    '''

    pass


def register_submodule_factory(module_name: str,
                               submodule_names: list) -> tuple:
    ''' Utility function to create register and unregister functions which simply load submodules, calling their register & unregister functions.

    :param module_name: The module name, typically __name__ .
    :type module_name: str
    :param submodule_names: List of submodule names to load and unload.
    :type submodule_names: list
    :rtype: tuple
    :return: register and unregister functions.
    '''

    pass


def register_tool(tool_cls,
                  *,
                  after: 'bpy.context.collection' = None,
                  separator: bool = False,
                  group: bool = False):
    ''' Register a tool in the toolbar.

    :param tool: A tool subclass.
    :type tool: 'bpy.types.WorkSpaceTool'
    :param space_type: Space type identifier.
    :type space_type: str
    :param after: Optional identifiers this tool will be added after.
    :type after: 'bpy.context.collection'
    :param separator: When true, add a separator before this tool.
    :type separator: bool
    :param group: When true, add a new nested group of tools.
    :type group: bool
    '''

    pass


def resource_path(type: str,
                  major: int = 'bpy.app.version[0]',
                  minor: str = 'bpy.app.version[1]') -> str:
    ''' Return the base path for storing system files.

    :param type: string in ['USER', 'LOCAL', 'SYSTEM'].
    :type type: str
    :param major: major version, defaults to current.
    :type major: int
    :param minor: minor version, defaults to current.
    :type minor: str
    :rtype: str
    :return: the resource path (not necessarily existing).
    '''

    pass


def script_path_pref():
    ''' returns the user preference or None

    '''

    pass


def script_path_user():
    ''' returns the env var and falls back to home dir or None

    '''

    pass


def script_paths(*,
                 subdir: str = None,
                 user_pref: bool = True,
                 check_all: bool = False,
                 use_user=True) -> list:
    ''' Returns a list of valid script paths.

    :param subdir: Optional subdir.
    :type subdir: str
    :param user_pref: Include the user preference script path.
    :type user_pref: bool
    :param check_all: Include local, user and system paths rather just the paths blender uses.
    :type check_all: bool
    :rtype: list
    :return: script paths.
    '''

    pass


def smpte_from_frame(frame: int, *, fps=None, fps_base=None) -> str:
    ''' Returns an SMPTE formatted string from the *frame*: HH:MM:SS:FF . If *fps* and *fps_base* are not given the current scene is used.

    :param frame: frame number.
    :type frame: int
    :rtype: str
    :return: the frame string.
    '''

    pass


def smpte_from_seconds(time: typing.Union[float, int],
                       *,
                       fps=None,
                       fps_base=None) -> str:
    ''' Returns an SMPTE formatted string from the *time*: HH:MM:SS:FF . If *fps* and *fps_base* are not given the current scene is used.

    :param time: time in seconds.
    :type time: typing.Union[float, int]
    :rtype: str
    :return: the frame string.
    '''

    pass


def time_from_frame(frame, fps, fps_base):
    ''' 

    '''

    pass


def time_to_frame(time, fps, fps_base):
    ''' 

    '''

    pass


def unescape_identifier(string: str) -> str:
    ''' Simple string un-escape function used for animation paths. This performs the reverse of escape_identifier .

    :param string: text
    :type string: str
    :rtype: str
    :return: The un-escaped string.
    '''

    pass


def unregister_class(cls):
    ''' Unload the Python class from blender. If the class has an *unregister* class method it will be called before unregistering.

    '''

    pass


def unregister_manual_map(manual_hook):
    ''' 

    '''

    pass


def unregister_tool(tool_cls):
    ''' 

    '''

    pass


def user_resource(resource_type, *, path: str = '',
                  create: bool = False) -> str:
    ''' Return a user resource path (normally from the users home directory).

    :param type: Resource type in ['DATAFILES', 'CONFIG', 'SCRIPTS', 'AUTOSAVE'].
    :type type: str
    :param path: Optional subdirectory.
    :type path: str
    :param create: Treat the path as a directory and create it if its not existing.
    :type create: bool
    :rtype: str
    :return: a path.
    '''

    pass
